/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
#define NUM_OF_CRC_DUMMY	10
/* Private function prototypes -----------------------------------------------*/
void CRC_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void CRC_Configure(void);
void DMA_Configure(void);
void CRC_CheckSumDMACalculationRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
" CRC demo \n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"\t - CRC Checksum DMA RAM (10 Samples)\n\r"
"************************************************\n\r";;
uint32_t crc_dummy_data[NUM_OF_CRC_DUMMY] = {0x12345678, 0x9ABCDEF1, 0x23456789, 0xABCDEF12, 0x3456789A, 0xBCDEF123, 0x456789AB, 0xCDEF1234, 0x56789ABC, 0xDEF12345};
DMA_Ch_CFG_Type DMACU1Tx;

/**********************************************************************
 * @brief		CRC_IRQHandler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void CRC_IRQHandler_IT(void)
{
	uint32_t status;
	
	status = HAL_CRC_GetState_IT();
	if((status & CRC_FLAG) == CRC_FLAG){
		HAL_CRC_GetState_IT();
		__NOP();
	}
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}





/**********************************************************************
 * @brief		CRC_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CRC_Configure(void)
{

  uint16_t 					chk_sum;	
	uint16_t 					chk_sum_init;
	uint32_t 					i;
	uint32_t 					j;
	
	if(HAL_CRC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
  /* Checksum calculation by Core*/
	chk_sum_init = 0;			/* CRC_INIT Value*/
	chk_sum = chk_sum_init;
	for(i=0;i<NUM_OF_CRC_DUMMY;i++){
		for (j=0; j<4; j++) {
			chk_sum += (uint16_t) (crc_dummy_data[i] >> (24-j*8)) & 0x00FF;
		}
	}

	#ifdef _DEBUG_MSG
	_DBG_(" ");
	_DBG("[Manual Calculation result] chk_sum = "); _DBH16(chk_sum); _DBG_(" "); 
	#endif
	
  /* Checksum calculation by CRC Block*/
	HAL_CRC_SetUserInput(chk_sum_init);
	HAL_CRC_ConfigInterrupt(ENABLE);
	HAL_CRC_ConfigUserMode(CRC_CHECKSUM, CRC_CCITT, CRC_MSB);
	NVIC_SetPriority(CRC_IRQn, 3);
  NVIC_EnableIRQ(CRC_IRQn);


	
}

/**********************************************************************
 * @brief		DMA_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void DMA_Configure(void)
{	

	
	if(HAL_DMAC_Init() != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
	DMACU1Tx.TransferCnt = NUM_OF_CRC_DUMMY;
	DMACU1Tx.PeriSel = DMAC_PERISEL_CRC;
	DMACU1Tx.TransferWidth = DMAC_WIDTH_WORD;
	DMACU1Tx.Dir = DMAC_DIR_TX;	
	DMACU1Tx.PeriAddr = CRC_BASE+4;
	DMACU1Tx.MemAddr = (uint32_t)(&crc_dummy_data[0]);	
	HAL_DMAC_Setup(DMA3, &DMACU1Tx);	
		
}


/**********************************************************************
 * @brief		CRC_CheckSumDMACalculationRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CRC_CheckSumDMACalculationRun(void)
{	
	
	uint16_t 					chk_sum;	
	
  CRCRun();
	HAL_DMAC_ChCmd(DMA3, ENABLE);
	
	while (HAL_DMAC_GetStatus(DMA3) == RESET) ;
	
	HAL_DMAC_ChCmd(DMA3, DISABLE);
	
	chk_sum = HAL_CRC_GetData();
	CRCStop();
	
	#ifdef _DEBUG_MSG
	_DBG("[CRC Block Calculation Result] chk_sum = "); _DBH16(chk_sum); _DBG_(" ");
	#endif
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*CRC Configure*/
	CRC_Configure();
	
  /*DMA Configure*/
	DMA_Configure();
	
	/* Enable IRQ Interrupts */
	__enable_irq();
	
  /* Calculation  Result */
	CRC_CheckSumDMACalculationRun();
	
	/* Infinite loop */
	while(1)
	{
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
