/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private function prototypes ---------------------------------------*/
void GPIOCD_IRQHandler_IT(void);
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void EXIT_Configure(void);
void SysTick_Configure(void);
void PWR_DeepSleepRun(void);
void PWR_SetPinLoop(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
uint32_t msec;
const uint8_t menu[] =
"************************************************\n\r"
"Sleep demo\n\r"
"\t - MCU: A31G21x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"************************************************\n\r";


/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler_IT (void) 					
{
	/* SysTick Interrupt Handler @ 1000Hz*/
	if(msec)msec--;
}


/**********************************************************************
 * @brief		GPIOE Handler_IT
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void GPIOCD_IRQHandler_IT(void)
{
	uint32_t	 status;
	
	status = HAL_GPIO_EXTI_GetState(PC);
	if (status & (1<<0))
	{
		HAL_GPIO_EXTI_ClearPin(PC, status&(1<<0));
	}
	
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	#ifdef _DEBUG_MSG
	_DBG(menu);
	#endif
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}



/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/* Test Pin setting PE0 */
	HAL_GPIO_ConfigOutput(PE, 0, PUSH_PULL_OUTPUT);
	HAL_GPIO_ConfigPullup(PE, 0, DISPUPD);	
	HAL_GPIO_ClearPin(PE, _BIT(0));

	/* External Test pin PA0*/
	HAL_GPIO_ConfigOutput(PA, 0, INPUT);
	HAL_GPIO_ConfigPullup(PA, 0, ENPU);
	HAL_GPIO_ClearPin(PA, _BIT(0));
	
	/* External Interrupt Test pin PC0*/
	HAL_GPIO_ConfigOutput(PC, 0, INPUT);
	HAL_GPIO_ConfigPullup(PC, 0, ENPU);
	HAL_GPIO_ClearPin(PC, _BIT(0));
}

/**********************************************************************
 * @brief		EXIT_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void EXIT_Configure(void)
{
	HAL_GPIO_EXTI_Config(PC, 0, IER_EDGE, 1);
	HAL_SCU_LVRCmd(DISABLE);
	
	NVIC_SetPriority(GPIOCD_IRQn, 3);
	NVIC_EnableIRQ(GPIOCD_IRQn);
	
	
	HAL_SCU_WakeUpSRCCmd(WAKEUP_GPIOC, ENABLE);
	
	
	SCU->SMR = 0
	| (SCU_SMR_ROSCAON_Msk)
	| (SCU_SMR_BGRAON_Msk)
	| (SCU_SMR_VDCAON_Msk)
	;
		
}


/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	/*1msec interrupt */
	SysTick_Config(SystemCoreClock/1000);  
	
}

/**********************************************************************
 * @brief		PWR_DeepSleepRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_DeepSleepRun(void)
{
	#ifdef _DEBUG_MSG
	_DBG("Press PA0 to Enter Deep Sleep Mode !");
	#endif
	while(HAL_GPIO_ReadPin(PA) & (1UL<<0));
	
	#ifdef _DEBUG_MSG
	_DBG("\n\rEnter Deep Sleep... Connect PC0 pin to GND to exit sleep mode");
	#endif
	
	// Change Clock to LSI(40KHz)
	HAL_SCU_SystemClockChange(SC_LSIOSC);
	
	HAL_SCU_LSE_ClockConfig(DISABLE);
	HAL_SCU_HSI_ClockConfig(DISABLE);
	HAL_SCU_HSE_ClockConfig(DISABLE);
	
	
	HAL_PWR_EnterPowerDownMode();
	
	
//	HAL_SCU_ClearWakeUpSRC(WAKEUP_GPIOE);
//	NVIC_ClearPendingIRQ(GPIOCD_IRQn);
	SystemClock_Config();
		
	#ifdef _DEBUG_MSG
	_DBG("\n\rWaked Up from Deep Sleep");
	#endif
}

/**********************************************************************
 * @brief		PWR_SetPinLoop
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void PWR_SetPinLoop(void)
{	
		msec = 1000;
		while(msec);
		
		HAL_GPIO_SetPin(PE, _BIT(0));
		
		msec = 1000;
		while(msec);
		
		HAL_GPIO_ClearPin(PE, _BIT(0));
		
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();

	/*EXIT Configure*/
	EXIT_Configure();
		
	/*Configure SysTick peripheral*/
	SysTick_Configure();

	/* Enable IRQ Interrupts */
	__enable_irq();
	
	/*PowerDownMode Run*/
	PWR_DeepSleepRun();

	while(1)
	{
		PWR_SetPinLoop();
   /* Infinite loop */
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing UART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
